Imports System.Data.SqlClient

Module DataStuff

    Private Const APP_TITLE = "OrderEntryTabs"

    Public Const CONNECT_STRING = _
        "data source=Bender\NETSDK;initial catalog=Orders;user id=sa"

    ' The main MDI form.
    Public g_MdiMain As mdiMain

    ' True when the data is modified.
    Public m_DataModified As Boolean

    ' Objects that work with the database.
    Public g_DataSet As New DataSet()

    ' DataTables.
    Public WithEvents g_dtCustomers As DataTable
    Public WithEvents g_dtCustomerOrders As DataTable
    Public WithEvents g_dtCustomerAddresses As DataTable
    Public WithEvents g_dtOrderItems As DataTable
    Public WithEvents g_dtOrderDates As DataTable

    ' SqlDataAdapters.
    Public g_daCustomers As SqlDataAdapter
    Public g_daCustomerOrders As SqlDataAdapter
    Public g_daCustomerAddresses As SqlDataAdapter
    Public g_daOrderItems As SqlDataAdapter
    Public g_daOrderDates As SqlDataAdapter

    ' CurrencyManagers.
    Public WithEvents g_cmCustomers As CurrencyManager

    ' Return the largest customer ID.
    Public Function MaxCustomerId() As Integer
        Dim data_view As New DataView(g_dtCustomers)

        ' See if there are any records.
        If data_view.Count < 1 Then
            ' There are no records. Return 0.
            Return 0
        Else
            ' There are records. Sort them.
            data_view.Sort = "CustomerId DESC"

            ' Get the row with the largest value.
            Dim data_row_view As DataRowView = data_view.Item(0)

            ' Return that value.
            Return data_row_view.Item("CustomerId")
        End If
    End Function

    ' Return the largest order ID.
    Public Function MaxOrderId() As Integer
        Dim data_view As New DataView(g_dtCustomerOrders)

        ' See if there are any records.
        If data_view.Count < 1 Then
            ' There are no records. Return 0.
            Return 0
        Else
            ' There are records. Sort them.
            data_view.Sort = "OrderId DESC"

            ' Get the row with the largest value.
            Dim data_row_view As DataRowView = data_view.Item(0)

            ' Return that value.
            Return data_row_view.Item("OrderId")
        End If
    End Function

    ' Return the largest address ID.
    Public Function MaxAddressId() As Integer
        Dim data_view As New DataView(g_dtCustomerAddresses)

        ' See if there are any records.
        If data_view.Count < 1 Then
            ' There are no records. Return 0.
            Return 0
        Else
            ' There are records. Sort them.
            data_view.Sort = "AddressId DESC"

            ' Get the row with the largest value.
            Dim data_row_view As DataRowView = data_view.Item(0)

            ' Return that value.
            Return data_row_view.Item("AddressId")
        End If
    End Function

    ' Load the data.
    Public Sub LoadData()
        ' Load the Customers table.
        LoadTable("Customers", _
            "SELECT CustomerName + ' (' + LTRIM(CustomerId) + ')' AS CombinedName, * FROM Customers ORDER BY CustomerName", _
            "", "", "", _
            g_dtCustomers, g_daCustomers)

        ' Load the CustomerOrders table.
        LoadTable("CustomerOrders", _
            "SELECT * FROM CustomerOrders", _
            "CustomerId", "Customers", "CustomerId", _
            g_dtCustomerOrders, g_daCustomerOrders)

        ' Load the CustomerAddresses table.
        LoadTable("CustomerAddresses", _
            "SELECT * FROM CustomerAddresses ORDER BY AddressType", _
            "CustomerId", "Customers", "CustomerId", _
            g_dtCustomerAddresses, g_daCustomerAddresses)

        ' Load the OrderItems table.
        LoadTable("OrderItems", _
            "SELECT * FROM OrderItems ORDER BY OrderId", _
            "OrderId", "CustomerOrders", "OrderId", _
            g_dtOrderItems, g_daOrderItems)

        ' Load the OrderDates table.
        LoadTable("OrderDates", _
            "SELECT * FROM OrderDates", _
            "OrderId", "CustomerOrders", "OrderId", _
            g_dtOrderDates, g_daOrderDates)
    End Sub

    ' Load a table into the DataSet.
    Private Sub LoadTable(ByVal table_name As String, ByVal select_statement As String, ByVal child_column_name As String, ByVal parent_table_name As String, ByVal parent_column_name As String, ByRef data_table As DataTable, ByRef data_adapter As SqlDataAdapter)
        ' Create a data adapter.
        data_adapter = New SqlDataAdapter( _
            select_statement, CONNECT_STRING)

        ' Create INSERT, UPDATE, and DELETE commands.
        Dim command_builder As New SqlCommandBuilder(data_adapter)

        ' Map the default table name "Table" to 
        ' the table's real name.
        data_adapter.TableMappings.Add("Table", table_name)

        ' Load the DataSet.
        data_adapter.Fill(g_DataSet)

        ' Save a reference to the new table.
        data_table = g_DataSet.Tables(table_name)

        ' Connect the tables with a foreign key constraint.
        If child_column_name.Length > 0 Then
            Dim parent_table As DataTable = g_DataSet.Tables(parent_table_name)
            Dim foreign_key As New ForeignKeyConstraint( _
                parent_table.Columns(parent_column_name), _
                data_table.Columns(child_column_name))
            data_table.Constraints.Add(foreign_key)
        End If
    End Sub

    ' Save changes to the database.
    Public Sub SaveData()
        ' End any current edits.
        EndAllEdits()

        ' See if there are any changes to save.
        If Not g_DataSet.HasChanges() Then Exit Sub

        ' INSERT.
        SaveTableChange(g_dtCustomers, g_daCustomers, DataRowState.Added)
        SaveTableChange(g_dtCustomerAddresses, g_daCustomerAddresses, DataRowState.Added)
        SaveTableChange(g_dtCustomerOrders, g_daCustomerOrders, DataRowState.Added)
        SaveTableChange(g_dtOrderItems, g_daOrderItems, DataRowState.Added)
        SaveTableChange(g_dtOrderDates, g_daOrderDates, DataRowState.Added)

        ' UPDATE.
        SaveTableChange(g_dtCustomers, g_daCustomers, DataRowState.Modified)
        SaveTableChange(g_dtCustomerAddresses, g_daCustomerAddresses, DataRowState.Modified)
        SaveTableChange(g_dtCustomerOrders, g_daCustomerOrders, DataRowState.Modified)
        SaveTableChange(g_dtOrderItems, g_daOrderItems, DataRowState.Modified)
        SaveTableChange(g_dtOrderDates, g_daOrderDates, DataRowState.Modified)

        ' DELETE.
        ' Note that OrderItems and OrderDates are only
        ' deleted when the Customers record is deleted.
        ' Then a CASCADE statement deletes them so we
        ' don't need to delete them ourselves.
        '   SaveTableChange(g_dtOrderDates, g_daOrderDates, DataRowState.Deleted)
        '   SaveTableChange(g_dtOrderItems, g_daOrderItems, DataRowState.Deleted)
        SaveTableChange(g_dtCustomerOrders, g_daCustomerOrders, DataRowState.Deleted)
        SaveTableChange(g_dtCustomerAddresses, g_daCustomerAddresses, DataRowState.Deleted)
        SaveTableChange(g_dtCustomers, g_daCustomers, DataRowState.Deleted)

        ' Mark the records as unmodified.
        g_DataSet.AcceptChanges()

        ' Update the form's caption to show the data is unmodified.
        CheckModified()
    End Sub

    ' Finish any current edits.
    Public Sub EndAllEdits()
        Dim frm As frmCustomer
        Dim child As Form
        For Each child In g_MdiMain.MdiChildren
            ' Only check frmCustomer forms.
            If child.GetType Is GetType(frmCustomer) Then
                frm = child
                frm.CustomerFormEndAllEdits()
            End If
        Next child

        ' End edits on the Customers table.
        g_cmCustomers.EndCurrentEdit()
    End Sub

    ' Save one kind of change to the table.
    Private Sub SaveTableChange(ByVal data_table As DataTable, ByVal data_adapter As SqlDataAdapter, ByVal data_row_state As DataRowState)
        Dim changes As DataTable

        changes = data_table.GetChanges(data_row_state)
        If Not (changes Is Nothing) Then
            Try
                data_adapter.Update(changes)
            Catch exc As Exception
                MsgBox(exc.Message)
            End Try
        End If
    End Sub

    ' Return True if the data is safe.
    Public Function DataSafe() As Boolean
        ' Finish any current edits.
        EndAllEdits()

        ' If there are no pending changes,
        ' the data is safe.
        If Not g_DataSet.HasChanges() Then Return True

        ' Ask the user if we should save the changes.
        Select Case MsgBox("Save changes?", MsgBoxStyle.YesNoCancel, "Save Changes?")
            Case MsgBoxResult.No
                ' The data is safe.
                Return True
            Case MsgBoxResult.Yes
                ' Save the changes.
                SaveData()

                ' Return False if we failed to save
                ' the data for some reason.
                Return (Not g_DataSet.HasChanges())
            Case MsgBoxResult.Cancel
                ' The user wants to cancel this operation.
                ' Do not let the program discard the data.
                Return False
        End Select
    End Function

    ' Flag the data as modified.
    Public Sub CheckModified()
        ' Do not recurse.
        Static running As Boolean
        If running Then Exit Sub
        running = True

        ' Make any pending changes register
        ' with the DataSet.
        EndAllEdits()

        ' See if the current indication is incorrect.
        If g_DataSet.HasChanges <> m_DataModified Then
            ' Update the indication.
            m_DataModified = g_DataSet.HasChanges
            If m_DataModified Then
                g_MdiMain.Text = APP_TITLE & " *"
            Else
                g_MdiMain.Text = APP_TITLE
            End If
            Beep()
        End If

        running = False
    End Sub
    Public Sub g_dtCustomers_ColumnChanged(ByVal sender As Object, ByVal e As System.Data.DataColumnChangeEventArgs) Handles g_dtCustomers.ColumnChanged
        CheckModified()
    End Sub
    Public Sub g_dtCustomerAddresses_ColumnChanged(ByVal sender As Object, ByVal e As System.Data.DataColumnChangeEventArgs) Handles g_dtCustomerAddresses.ColumnChanged
        CheckModified()
    End Sub
    Public Sub g_dtCustomerOrders_ColumnChanged(ByVal sender As Object, ByVal e As System.Data.DataColumnChangeEventArgs) Handles g_dtCustomerOrders.ColumnChanged
        CheckModified()
    End Sub
    Public Sub g_dtOrderItems_ColumnChanged(ByVal sender As Object, ByVal e As System.Data.DataColumnChangeEventArgs) Handles g_dtOrderItems.ColumnChanged
        CheckModified()
    End Sub
    Public Sub g_dtOrderDates_ColumnChanged(ByVal sender As Object, ByVal e As System.Data.DataColumnChangeEventArgs) Handles g_dtOrderDates.ColumnChanged
        CheckModified()
    End Sub

    ' Cancel the changes.
    Public Sub CancelChanges()
        ' Finish the current edit.
        EndAllEdits()

        ' See if there are any changes to cancel.
        If g_DataSet.HasChanges() Then
            ' Make the user confirm.
            If MsgBox("Discard changes?", _
                MsgBoxStyle.YesNo Or MsgBoxStyle.Question, _
                "Discard changes?") = MsgBoxResult.No Then Exit Sub

            ' Cancel the changes.
            g_DataSet.RejectChanges()
        End If

        ' Update the form's caption to show 
        ' the data is unmodified.
        CheckModified()
    End Sub

End Module
